//    Peek, An Image Viewing Program for BeOS
//    Copyright (C) 2000 Durand Miller
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, version 2.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "PeekApp.h"

// PeekApp - Loads the configuration, makes a new window,
//           initializes the two windows
PeekApp::PeekApp() : BApplication("application/x-vnd.Peek") {
 Setup = new PeekConfiguration;
 LoadConfiguration(Setup);
 BRect *box = new BRect(Setup->Frame());
 myWindow = new PeekWindow(*box,"Peek",Setup);
 optionsWindow = NULL;
 delete box;
 loadedViaTracker = false;
}

// RefsReceived - Takes the first ref we get and tells the window to load it.
void PeekApp::RefsReceived(BMessage *e) {
 entry_ref *entryLoad = new entry_ref;
 if (e->FindRef("refs",entryLoad) == B_OK) {
   if (Setup->OpenCompact() && IsLaunching()) {                                    // This is "compact" mode, for ShowImage-niceness.
     loadedViaTracker = true;
     Setup->SetViewMode(P_WINDOW);
     if (!Setup->FileListHide()) myWindow->ToggleFileList();
     myWindow->MoveTo(50,50);
     myWindow->ResizeTo(100,100);  // Less than this and it conflict with the window's max * min width/heights.. which results in a wonky setting of the scroll bar values
    } 

   myWindow->LoadImage(entryLoad);
 }
 delete entryLoad;
}

// LoadConfiguration - Loads "Peek_settings" from the config directory
//                     and returns them in the PeekConfiguration object *S
void PeekApp::LoadConfiguration(PeekConfiguration* S) {
 BPath *settingsPath = new BPath();

 if (find_directory(B_USER_SETTINGS_DIRECTORY,settingsPath) == B_OK) {
   BDirectory *settingsDirectory = new BDirectory(settingsPath->Path());
   BEntry *settingsEntry = new BEntry();

   if ( settingsDirectory->FindEntry("Peek_settings",settingsEntry) == B_NO_ERROR){
      BFile *settingsFile = new BFile(settingsEntry,B_READ_ONLY);
      settingsFile->Read(Setup,sizeof(PeekConfiguration));
      delete settingsFile;

      char ver[10];
      S->GetVersion(ver);
      if (strcmp(ver,PeekVersion)!=0) SetDefaults(S);

   } else SetDefaults(S);
    
   delete settingsEntry;
   delete settingsDirectory;
 } else SetDefaults(S);

 delete settingsPath;
}

// SaveConfiguration - Saves the PeekConfiguration *S into "Peek_settings" in the config directory.
//                     It just uses a straight memory dump.
void PeekApp::SaveConfiguration(PeekConfiguration* S) {
 BPath *settingsPath = new BPath();

 if (find_directory(B_USER_SETTINGS_DIRECTORY,settingsPath) == B_OK) {
   BDirectory *settingsDirectory = new BDirectory(settingsPath->Path());
   BEntry  *settingsEntry = new BEntry();

   if ( settingsDirectory->FindEntry("Peek_settings",settingsEntry) == B_NO_ERROR){
      BFile *settingsFile = new BFile(settingsEntry,B_WRITE_ONLY);
      settingsFile->Write(Setup,sizeof(PeekConfiguration));
      delete settingsFile;
   } else {
      BFile *settingsFile = new BFile(); 
      settingsDirectory->CreateFile("Peek_settings",settingsFile);
      settingsFile->Write(Setup,sizeof(PeekConfiguration));
      delete settingsFile;
   };
    
   delete settingsEntry;
   delete settingsDirectory;
 }

 delete settingsPath;
}

// SetDefaults - Sets *s to all the defaults (the options I think really)
void PeekApp::SetDefaults(PeekConfiguration* s) {
  s->SetVersion(PeekVersion);
  s->SetFullScreen(false);
  s->SetSlideshowMode(P_DESCEND);
  s->SetSlideshowDelay(2000000);
  s->SetLeftButton(P_SCROLLING);
  s->SetRightButton(P_DRAGGING);
  s->SetMiddleButton(P_CLIPPING);
  s->SetViewMode(P_NORMAL);  
  s->SetDragMode(B_OP_OVER);               // B_OP_OVER is what Peek uses to signify the ICON drag mode. B_OP_OVER is never really used.
  s->SetFrame(BRect(50,50,550,500));       // Where the window is displayed originally.
  s->SetFileDrop(false);
  s->SetChangeDirectory(true);
  s->SetRunFile(true);
  s->SetDeleteAll(false);
  s->SetChangeOnDesktopLoad(true);
  s->SetFileListRight(150);                // 150 is the position of the right-hand side of the file list view.
  s->SetFileListHide(false);
  s->SetOpenCompact(true);
}

// ShowOptions - if the options window is around, make it active. If not, spawn another one.
void PeekApp::ShowOptions() {
 if (optionsWindow != NULL) { 
   optionsWindow->Activate();
   return;
 }  
 BRect *box = new BRect(Setup->Frame().left+10,Setup->Frame().top+10,Setup->Frame().left + 460,Setup->Frame().top+360);
 optionsWindow = new OptionsWindow(*box,"Peek Options",Setup);
 optionsWindow->Show();
 delete box;
}


// MessageReceived 
void PeekApp::MessageReceived(BMessage *e) {
  switch (e->what) {
   // node monitor
  case B_NODE_MONITOR:		myWindow->RefreshFileView(); break;
   // others
   case PEEK_OPTIONS_WINDOW: ShowOptions(); break;
   case PEEK_OPTIONS_CLOSE: SaveConfiguration(Setup); 
                            optionsWindow = NULL; 
                            break;
   case PEEK_FULL_SCREEN: myWindow->PostMessage(e); break;
   case PEEK_SAVE_FILE: myWindow->PostMessage(e); break;
   case PEEK_FILES_HIDE: myWindow->PostMessage(e); break;
   default: BApplication::MessageReceived(e); break;
  }
}

// AboutRequested - This just spawns an about box.
//                  REMEMBER TO CHANGE THE VERSION NUMBER HERE
void PeekApp::AboutRequested() {
 char aboutMsg[255];
 strcpy(aboutMsg,PeekVersion);
 strcat(aboutMsg, "\nCopyright (C) 2000 Durand Miller\nLicensed under the GPL.\n\nBug reports, patches, fixes, suggestions and comments are very welcome.\n\nbepeek@attglobal.net\n\nhttp://pws.prserv.net/durand/peek/");
 (new BAlert("AboutBox",aboutMsg,"OK"))->Go();
}

// Quit - Save the Setup object if we need to, destroy it and then return to nothing.
void PeekApp::Quit() {
 if (!loadedViaTracker) SaveConfiguration(Setup);   // only save if Peek was started individually. Is this a good idea?  Most probably Peek won't be started individually.
 delete Setup;
 BApplication::Quit();
}

// ReadyToRun - put here to stop that flickering of the old window (just before RefsReceived changes it) 
//              when loading up.
void PeekApp::ReadyToRun() {
 myWindow->Show();
}