#include "imageView.h"
#include <Resources.h>

imageView::imageView(BRect R,char* name, PeekConfiguration* s, BMessage *n) 
: BView(R,name,B_FOLLOW_ALL_SIDES,B_WILL_DRAW | B_FRAME_EVENTS | B_FULL_UPDATE_ON_RESIZE)
{
  Setup = s;
  negotiationMessage = n;
  SetViewColor(0,0,0,0);
  imagePic = BTranslationUtils::GetBitmap(name);
  mouseDownFlag = false;
  clipRegion = NULL;
  mouseClipping = false;
}

imageView::~imageView() {
 if (imagePic) delete imagePic;
 if (currentRef) delete currentRef;
}

// Attached to window
void imageView::AttachedToWindow() {
 SetViewBitmap(imagePic);
 delete imagePic;
 imagePic = NULL;
 currentRef = NULL;
 BView::AttachedToWindow();
}

// Draw
void imageView::Draw(BRect R)  {
 if (imagePic == NULL) return;

 float imageWidth = imagePic->Bounds().right;
 float imageHeight = imagePic->Bounds().bottom;

 if (Setup->ViewMode() == P_TILED) 
   for (float left = 0; left < Bounds().Width(); left += imageWidth )
    for (float top = 0; top < Bounds().Height(); top += imageHeight ) {
     MovePenTo(left,top);
     DrawBitmap(imagePic);
    } 

 float left,top;
 if (imageWidth > Bounds().Width()) left = 0; else left = ( (Bounds().Width() - imageWidth) / 2);
 if (imageHeight > Bounds().Height()) top = 0; else top = ( (Bounds().Height() - imageHeight)/2);

 if (Setup->ViewMode() == P_NORMAL || Setup->ViewMode() == P_WINDOW) {
   MovePenTo(left,top);
   DrawBitmap(imagePic);
 }

 if (clipRegion != NULL) {
   StrokeRect(*clipRegion, B_MIXED_COLORS);
 }

 Sync();
}


// size of image
void imageView::RequestedSize(float *x, float *y) {
  if (!imagePic) {
   *x = 0;
   *y = 0;
  } else {
    *x = imagePic->Bounds().right;
    *y = imagePic->Bounds().bottom;
  }  
}

// GetBitmap() 
BBitmap* imageView::GetBitmap() {
  return imagePic;
}

// GetFilename()
int imageView::GetFilename(char* fileName) {
 if (imagePic == NULL) return B_ERROR;
 if (currentRef == NULL) return B_ERROR;
 BEntry *tempEntry = new BEntry(currentRef);
 BPath *tempPath = new BPath;
 tempEntry->GetPath(tempPath);
 strcpy(fileName,tempPath->Path());
 delete tempEntry;
 delete tempPath;
 return B_OK;
}

void imageView::Clear() {
 if (imagePic) {
    delete imagePic;
    imagePic = NULL;
   }
 if (currentRef) {
    delete currentRef;
    currentRef = NULL;
   }
}

// GetRef(entry_ref *)
int imageView::GetRef(entry_ref* file) {
 if (currentRef == NULL) return B_ERROR;
 *file = *currentRef;
 return B_OK;
}

// loadImage (entry_ref)
int imageView::loadImage(entry_ref fileRef) {
 Clear();
 Hide();
 imagePic = BTranslationUtils::GetBitmap(&fileRef);
 if (imagePic) {
   currentRef = new entry_ref(fileRef);
   Show();
   return 0;
 }  
 Show();
 return 1;
}

// loadImage (BEntry *)
int imageView::loadImage(BEntry *e) {
  entry_ref *tempRef = new entry_ref;
  e->GetRef(tempRef);
  int k = loadImage(*tempRef);
  delete tempRef;
  return k;
}


// loadImage for BITMAPS
int imageView::loadImage(BBitmap *src) {
 Clear();
 Hide();

 imagePic = src;
 if (imagePic) {
   Show();
   return 0;
 }  
 Show();
 return 1;
}

// Mouse down
void imageView::MouseDown(BPoint a) {
  BPoint dummy,imagePoint;
  uint32 button;
  GetMouse(&dummy,&button);

  uint32 task = P_SCROLLING;
  if (button == B_PRIMARY_MOUSE_BUTTON) task = Setup->LeftButton();
  if (button == B_SECONDARY_MOUSE_BUTTON) task = Setup->RightButton();
  if (button == B_TERTIARY_MOUSE_BUTTON) task = Setup->MiddleButton();

  if (task == P_SCROLLING) {
    mouseDownFlag = true;
    mousePosition = new BPoint(a);  
    SetMouseEventMask(1);   // Receive MouseMoved even outside of BView
    return;
  };

  if (task == P_DRAGGING) {  
    if (imagePic == NULL) return;
    BBitmap *dragImage;
    BMessage *msg;

      
    // here comes the CTRL keyboard modifier
    if (modifiers() & B_CONTROL_KEY) {

      if (!Setup->FileDrop()) {
        msg = new BMessage(B_SIMPLE_DATA);
        if (currentRef) msg->AddRef("refs",currentRef);
       } else msg = new BMessage (*negotiationMessage);

    } else {

      if (Setup->FileDrop()) {
        msg = new BMessage(B_SIMPLE_DATA);
        if (currentRef) msg->AddRef("refs",currentRef);
       }  else msg = new BMessage (*negotiationMessage);

    }
       
    
    if (Setup->DragMode()!=B_OP_OVER) {
       if (imagePic->Bounds().Width() < Bounds().Width())  imagePoint.x = a.x - Bounds().Width()/2 +  imagePic->Bounds().Width()/2; else imagePoint.x = a.x;
       if (imagePic->Bounds().Height() < Bounds().Height()) imagePoint.y = a.y - Bounds().Height()/2 + imagePic->Bounds().Height()/2; else imagePoint.y = a.y;
       dragImage = new BBitmap(imagePic);
       DragMessage(msg,dragImage, Setup->DragMode(), imagePoint);
     } else {
        imagePoint.x = 32;
        imagePoint.y = 32;
        BMimeType k("image");          
        dragImage = new BBitmap(BRect(0,0,31,31),B_CMAP8);
        status_t h = k.GetIcon(dragImage,B_LARGE_ICON);
        if (h == B_OK) DragMessage(msg,dragImage, B_OP_ALPHA, imagePoint);
     }  
   
   delete msg;
   return;
  }

 if (task == P_CLIPPING) {
   if (clipRegion) {
      SetDrawingMode(B_OP_INVERT);
      StrokeRect(*clipRegion, B_MIXED_COLORS);
      SetDrawingMode(B_OP_COPY);
      delete clipRegion;
    }  
   if (imagePic == NULL) return;
   if (!InView(a)) return;
   mouseClipping = true;
   clipRegion = new BRect;
   clipRegion->Set(a.x,a.y,a.x,a.y); 
   SetMouseEventMask(1);   // Receive MouseMoved even outside of BView
   return;
 }
 
}

bool imageView::InView(BPoint a) {
 float imageWidth = imagePic->Bounds().Width();
 float imageHeight = imagePic->Bounds().Height();
 float left,top;
 if (imageWidth > Bounds().Width()) left = 0; else left = ( (Bounds().Width() - imageWidth) / 2);
 if (imageHeight > Bounds().Height()) top = 0; else top = ( (Bounds().Height() - imageHeight)/2);

 if (a.x > left && a.y > top && 
     a.x < imageWidth + left && a.y < imageHeight + top) return true;

 return false;
}

void imageView::DrawClipping() {
 if (clipRegion == NULL) return;
}

// mouse up
void imageView::MouseUp(BPoint a) {
  if (mouseDownFlag) {
    mouseDownFlag = false; 
    SetMouseEventMask(0);   // Stop the receive MouseMoved outside of BView
    delete mousePosition;
  }

  if (mouseClipping) {
    mouseClipping = false;
  }
}

// mouse moved
void imageView::MouseMoved(BPoint a, uint32 transit, const BMessage* e) {
  if (mouseDownFlag) {
   float hmin,hmax,vmin,vmax;
   ScrollBar(B_HORIZONTAL)->GetRange(&hmin,&hmax);  
   ScrollBar(B_VERTICAL)->GetRange(&vmin,&vmax);  
   float xdiff =  (mousePosition->x  - a.x) + ScrollBar(B_HORIZONTAL)->Value();
   float ydiff =  (mousePosition->y  - a.y) + ScrollBar(B_VERTICAL)->Value();
   if (xdiff <= hmin ) xdiff = hmin;
   if (ydiff <= vmin ) ydiff = vmin;   
   if (xdiff >= hmax ) xdiff = hmax;
   if (ydiff >= vmax ) ydiff = vmax;   
 
   ScrollTo(xdiff,ydiff);
   if (clipRegion) {
    clipRegion->left += (mousePosition->x  - a.x);
    clipRegion->right += (mousePosition->x  - a.x);
    clipRegion->top += (mousePosition->y  - a.y);
    clipRegion->bottom += (mousePosition->y  - a.y);
   }
  }

 if (mouseClipping) {
   SetDrawingMode(B_OP_INVERT);
   StrokeRect(*clipRegion, B_MIXED_COLORS);
   clipRegion->right = a.x;
   clipRegion->bottom = a.y;
   StrokeRect(*clipRegion, B_MIXED_COLORS);
   SetDrawingMode(B_OP_COPY);
  }
}


// supported format
bool imageView::SupportedFormat(const char *s) {
      translator_id *list;
      int32 num;
      BTranslatorRoster::Default()->GetAllTranslators(&list,&num);

       for (int i=0; i < num; i++) {
         int32 numOfTypes;
         const translation_format *in_types;
         BTranslatorRoster::Default()->GetInputFormats(list[i],&in_types,&numOfTypes);
         for (int j = 0; j <numOfTypes; j++) if ( strcmp(in_types[j].MIME,s) == 0) return true;
       }
 return false;
}

// handle mime
void imageView::HandleMime(BMessage *e) {
    int32 numBytes;
    type_code type;
    const void *data;
    const char *m_type;
    if (e->GetInfo( B_MIME_TYPE, 0, (char**)&m_type, &type, &numBytes) != B_OK) return;
    e->FindData(m_type,B_MIME_TYPE,&data,&numBytes);
    BMemoryIO memStream(data, numBytes);
    BBitmapStream imageStream;
    if (strcmp(m_type,"image/x-be-bitmap")==0) {
     uint8 buffer[1024];
     int32 size;
     while ((size=memStream.Read(buffer,1024))>0) 
       imageStream.Write(buffer,size);
    } 
    else if ( BTranslatorRoster::Default()->Translate(&memStream,NULL,NULL, &imageStream, B_TRANSLATOR_BITMAP, 0, m_type) != B_OK) return;

    BBitmap *bmp;
    imageStream.DetachBitmap(&bmp);
    loadImage(bmp);
    return;  
}

// choose format
void imageView::ChooseFormat(BMessage *e) {
                                           // CASE 1:  we have a file
     entry_ref ref;
     if (e->FindRef("refs", &ref) == B_OK )  {
       if (loadImage(ref)) {
        BMessage *changer = new BMessage(PEEK_CHANGE_DIRECTORY);
        changer->AddRef("directory",&ref);
        Window()->PostMessage(changer);
        delete changer;
       } else {
          if (Setup->ChangeDirectory()) {
            BEntry *dropped = new BEntry(&ref,true);
            dropped->GetParent(dropped);
            entry_ref *dropdir = new entry_ref;
            dropped->GetRef(dropdir);
            delete dropped;
            BMessage *changer = new BMessage(PEEK_CHANGE_DIRECTORY);
            changer->AddRef("directory",dropdir);
            delete dropdir;
            Window()->PostMessage(changer);
            delete changer;
          }
          BMessage *changer = new BMessage(PEEK_SLIDER_FIX);
          Window()->PostMessage(changer);
          delete changer;
         } 
       return;
     }

     const char *m_type;
     int32 i = 0, action;
     while ( e->FindInt32("be:actions",i,&action) == B_OK && action != B_COPY_TARGET) i++;
     if (action != B_COPY_TARGET) return;
     i = 0;
     while (e->FindString("be:types",i,&m_type) == B_OK && !SupportedFormat(m_type)) i++;
     if (!m_type) return; 
     
     BMessage yahoo(B_COPY_TARGET);
     yahoo.AddString("be:types",m_type);
     e->SendReply(&yahoo,this);
}

// message received
void imageView::MessageReceived(BMessage *e) {
  switch (e->what) {
    case B_MIME_DATA: HandleMime(e); break;
    case B_COPY_TARGET: ActionCopy(e); break;
    case B_TRASH_TARGET: Trash(); break;
    case B_SIMPLE_DATA: ChooseFormat(e); break;
    default: BView::MessageReceived(e);
  }
}

// trash
void imageView::Trash() {
  BMessage killer(PEEK_FILE_DELETE);
  Window()->PostMessage(&killer);
}

void imageView::ActionCopy(BMessage *e) {   
  entry_ref targetRef;
  // memory or file transfer ?
  if (e->FindRef("directory",&targetRef) == B_OK)  {

      BDirectory *targetDir = new BDirectory( &targetRef );
      if (targetDir->InitCheck() == B_OK) {
         BFile *targetFile = new BFile(targetDir,e->FindString("name"),B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
         if (targetFile->InitCheck() == B_OK) {
          BBitmapStream *strm = new BBitmapStream(imagePic);

          translator_id trans_id;
          uint32 type;

          GetTranslatorInfo(e->FindString("be:filetypes"),&trans_id,&type);
          BTranslatorRoster::Default()->Translate(trans_id,strm,NULL,targetFile,type);
          BNodeInfo *ni = new BNodeInfo(targetFile);
          ni->SetType(e->FindString("be:filetypes"));
          strm->DetachBitmap(&imagePic);
          delete ni;
          delete strm;
         } 
        delete targetFile;
       }   
     delete targetDir;
     return;
   } 
   
  else {
     BMallocIO *data = new BMallocIO;
     BBitmapStream *imageSrc = new BBitmapStream(imagePic);
     BMessage *package = new BMessage(B_MIME_DATA);
     translator_id trans_id;
     uint32 type;

     GetTranslatorInfo(e->FindString("be:types"),&trans_id,&type);
     if (BTranslatorRoster::Default()->Translate( trans_id, imageSrc, NULL, data, type ) != B_OK) return;
     package->AddData( e->FindString("be:types"),  B_MIME_DATA, data->Buffer(), data->BufferLength());
     e->SendReply(package);        
     imageSrc->DetachBitmap(&imagePic);
     delete data;
     delete imageSrc;
     delete package;
  } 

}


void imageView::GetTranslatorInfo(const char *m, translator_id *t, uint32 *t2) {
      translator_id *list;
      int32 num;
      BTranslatorRoster::Default()->GetAllTranslators(&list,&num);

       for (int i=0; i < num; i++) {
         int32 numOfTypes;
         const translation_format *out_types;
         BTranslatorRoster::Default()->GetOutputFormats(list[i],&out_types,&numOfTypes);
         for (int j = 0; j <numOfTypes; j++) if ( strcmp(out_types[j].MIME,m) == 0) {
           *t = list[i];
           *t2 = out_types[j].type;
           return;
         }
       }
 *t = 0;
 *t2 = 0;
}
